/*
 * tasks.c
 *
 * This software is provided as-is.  Lantronix and the author disclaim to the fullest 
 * extent authorized by law any and all warranties, whether express or implied, 
 * including, without limitation, any implied warranties of title, non-infringement, 
 * merchantability or fitness for a particular purpose.   Neither the author nor 
 * Lantronix are responsible for any damage or losses of any kind caused by the use or 
 * misuse of this software.  IN NO EVENT SHALL THE AUTHOR OR LANTRONIX BE 
 * LIABLE FOR ANY DAMAGES, INCLUDING  INCIDENTAL OR CONSEQUENTIAL 
 * DAMAGES, IN CONNECTION WITH OR ARISING FROM THE FURNISHING, 
 * PERFORMANCE, OR USE OF THIS SOFTWARE.
 * 
 */
#include <mqx.h>
#include <bsp.h>
#include <fio.h>
#include "main.h"

extern MQX_FILE_PTR InitializeIO(void);
extern void SetOutput(int signal,boolean state);

MQX_FILE_PTR ser_dev, ser_conf = NULL;
UART_SEM uartsem;

/*
 * This task sets up the serial ports of the Kinetis part to communicate to the
 * xPico Wi-Fi. It sets up the ADC to get the data from the potentiometer.
 * 
 * Also sets up a semaphore to keep the receive task and the send task from 
 * stepping on each other while using the serial port for data.
 * 
 */
void init_task(uint_32 initial_data) 
{
	_task_id task_id;
	ser_dev = InitializeIO();
	ser_conf = fopen("ittye:", NULL);
	
	_lwsem_create(&uartsem.READ_SEM, 1);

	task_id = _task_create(0, RECV_TASK, 0);

	_time_delay(3000);
	_task_create(0,CONFIG_TASK,0);
	task_id = _task_create(0, SEND_TASK, 0);
}

/*
 * The Config task reads the current status (XSR) of the xPico Wi-Fi on
 * the second serial port. This will return such things as SSID of the
 * infrastructure network, RSSI, IP address, etc. It polls every 30 seconds
 * and updates the global variables for reporting.
 * 
 */
void config_task(uint_32 initial_data)
{
	while (TRUE)
	{
		if (strcmp(uid,"00000000000000") == 0)
		{
			getXSR(ser_conf, "device");
		}
		getXSR(ser_conf, "wlan");
		_time_delay(30000);
		
	}
}

/*
 * The Receive task listens on the primary serial port for commands.
 * Depending on the command, it toggles an LED on the Kinetis board, then 
 * sends the current status of the xPico Wi-Fi.
 */
void recv_task(uint_32 initial_data)
{
	int_32 c;
	
	while (TRUE)
	{
		_lwsem_wait(&uartsem.READ_SEM);
		if (fstatus(ser_dev))
		{
			c = fgetc( ser_dev );
			if (c == IO_ERROR)
				break;
			switch (c) {
			case '1':
				ToggleLED(1);
				SendStatus(ser_dev);
				break;
			case '2':
				ToggleLED(2);
				SendStatus(ser_dev);
				break;
			case '3':
				ToggleLED(3);
				SendStatus(ser_dev);
				break;
			case '4':
				ToggleLED(4);
				SendStatus(ser_dev);
				break;
			case 'R':
				SendStatus(ser_dev);
			}
		}
		_lwsem_post(&uartsem.READ_SEM);
		_time_delay(100);
	}
}

/*
 * The Send task monitors the value of the potentiometer.
 * When it changes by more than 200, it creates an HTTP POST
 * message to send to the Cloud.
 */
void send_task(uint_32 initial_data)
{
	char buffer[1024];
	int size, result;
	ADC_VALUE_TYPE adcValue, prevValue = 100000;
	
	char data[12];
	char port[] = "Potentiometer";
	while (strcmp(uid, "00000000000000") == 0) {
		_time_delay(500);
	}
	while (TRUE)
	{
		adcValue = getADC();
		if (adcValue > 32768) 
			adcValue -= 65535;
		if (prevValue == 100000) {
			prevValue = adcValue;
		} else if (abs(adcValue - prevValue) > 200) {
			prevValue = adcValue;
			sprintf(data, "%d", adcValue);
			printf("Potentiometer changed to: %s \n", data);
			size = createPostMessage(buffer, data, port, uid);
			_lwsem_wait(&uartsem.READ_SEM);
			SendPost(ser_dev, buffer, size );
			result = waitPostResponse(ser_dev);
			printf("Done with response: %d\n", result);
			_lwsem_post(&uartsem.READ_SEM);
		}
		_time_delay(2000);
	}
}

